/*#############################################################################
# IPXSPX.H
#
# 18.05.1997 (Codor)
#
# Use swab for swapping bytes.
#############################################################################*/

#ifndef __IPXSPX_H__
#define __IPXSPX_H__

#include "types.h"

typedef void (cdecl far *TIpxSpxEntryPoint)(void);

#define SOCK_SHORTLIVED   0x00
#define SOCK_LONGLIVED    0xFF

//=============================================================================
// Utility functions
//=============================================================================

// Returns TIpxStatus::ipxReturnCode description
char *IpxGetErrorDesc(int errorcode);

// Swap functions
word Swap(word from);
long Swap(long from);

// Blank memory buffer
void ZeroMemory(void *address, word length);

//=============================================================================
// struct TIpxAddress
//
//=============================================================================

struct TIpxAddress {
  // Always set 0 (source and destination are on the same physical network)
  // High-low
  byte Network[4];

  // Physical address of the node (unique number of the Ethernet card)
  // If it is 0xFFFFFFFFFFFF then this is a broadcast packet when it is
  // transmitted
  // High-low
  byte Node[6];

  // Should use a value above 0x4000 and below 0x8000
  // High-low
  byte Socket[2];

public:
  TIpxAddress(bool isbroadcast = False);
  void GetLocalAddress(void);
  bool IsBroadcast(void);
  bool IsSame(TIpxAddress *addr, bool useSocket = False);
};

//=============================================================================
// struct TIpxHeader
// All numeric fields in TIpxHeader are high-low (MS byte in a field is the
// first one). Intel processors have a low-high organization.
//=============================================================================

struct TIpxHeader {
  // IPX will set this field to 0xFFFF
  // High-low
  word Checksum;

  // Length of complete packet (30 for this header + max 546 for data)
  // IPX will set this field
  // High-low
  word Length;

  // IPX will set this field to 0 before sending the packet
  byte TransportControl;

  // Use always 4 for Packet Exchange Packet (IPX)
  byte PacketType;

  // Specifies destination station
  TIpxAddress Dest;

  // Specifies source station
  TIpxAddress Src;

public:
  TIpxHeader();
  void SetDestAddress(TIpxAddress *dest);
  void SetSrcAddress(TIpxAddress *src);
};

//============================================================================
// struct TEventControlBlock
//
//============================================================================

// Borland C++ v3.1 uses AX register to load the correct DS (_loadds). Since
// EventServiceRoutine receives the caller ID in AL, it is unavailable.
// _ES:_SI contains a far pointer to the ECB.
typedef void (_cdecl _far _loadds *TEventServiceRoutine)(void);

struct TEventControlBlock {
  // Used internally by IPX only while InUse is != 0
  void far *LinkAddress;

  // May be NULL
  TEventServiceRoutine EventServiceRoutine;

  // Different from 0 while IPX is working
  byte InUse;

  // Contains a return code from IPX
  byte CompletionCode;

  // High-low. Must be opened
  word SocketNumber;

  // Used internally. Must not be modified
  byte IPXWorkSpace[4];
  byte DriverWorkspace[12];

  // Set to destination node if there are no bridges on the network
  // High-low
  byte ImmediateAddress[6];

  // Always set to 2 since there are 2 Frament(s)
  word FragmentCount;

  // Fragment[0] specifies the buffer and length for the IPX header (30 bytes)
  // Fragment[1] specifies the buffer and length for the IPX data (546 bytes)
  struct {
    void far *Address;
    word Length;
  } Fragment[2];

public:
  TEventControlBlock();
};

//=============================================================================
// class TIpxStatus
//
//=============================================================================

class TIpxStatus {
  public:
    enum ipxReturnCode {
      IpxSuccess = 0,
      IpxSocketTableIsFull,
      IpxSocketAlreadyOpen,
      IpxSocketNotOpen,
      IpxNoPathToDestination,
      IpxCannotCancelEcb,
      IpxEcbIsInUse,
      IpxEcbIsNotInUse,
      IpxDataTooLong,
      IpxAlreadyInUse,
    };
    enum ecbCompletionCode {
      EcbOk = 0,
      EcbEventCanceled = 0xFC,
      EcbPacketMalformed = 0xFD,
      EcbUndeliverablePacket = 0xFE,
      EcbNetworkFailure = 0xFF,
      EcbBufferTooSmall = 0xFD,
      EcbSocketNotOpen = 0xFF,
      EcbCannotCancel = 0xF9
    };
    enum ecbInUse {
      EcbInUseFree = 0x00,
      EcbInUseProcessing = 0xFA,
      EcbInUseWaiting = 0xFB,
      EcbInUseWaitingForAESTimeout = 0xFC,
      EcbInUseWaitingForTimeout = 0xFD,
      EcbInUseListening = 0xFE,
      EcbInUseSending = 0xFF
    };
};

//=============================================================================
// class TIpxSocket
//
//=============================================================================

class TIpxSocket {
  private:
    bool Created;
    word Longevity;
    word Socket;
    word ErrorCode;       // IPX.. funtions error code
    word ExErrorCode;     // One of enums from TIpxStatus
  public:
    TIpxSocket(word socket = 0,
               bool create = True, word longevity = SOCK_SHORTLIVED);
    ~TIpxSocket();
    word Create(void);
    void Destroy(void);
    word GetSocket(void);
    word GetLowHighSocket(void);
    word GetErrorCode(void);
};

//=============================================================================
// class TIpx
//
//=============================================================================

class TIpx {
  private:
    word ErrorCode;       // IPX.. functions error code
    word ExErrorCode;     // One of enums from TIpxStatus
    char Data[546];
    word DataLength;
    TIpxHeader Header;
    TEventControlBlock Ecb;
    TIpxSocket *SocketPtr;
  public:
    TIpx(TIpxSocket *socket = NULL,
         TEventServiceRoutine proc = NULL, word dataSize = 546);
    ~TIpx();
    void SetSocket(TIpxSocket *socket);
    void SetDestination(TIpxAddress *dest, TIpxSocket *socket);
    int Send(void *data = NULL, int length = 0);
    int Listen(void);
    int Cancel(void);
    int GetData(void far *data, int length);
    TIpxHeader far *GetHeader(TIpxHeader *hdr = NULL) const;
    TEventControlBlock far *GetEcb(TEventControlBlock *ecb = NULL) const;
    word GetErrorCode(void);
    bool IsCompleted(void);
    bool IsFree(void);

  static bool Init(void);
};

//=============================================================================
// struct TNetworkAddress
// struct TNodeAddress
// Partial network addresses
//=============================================================================

struct TNetworkAddress {
  byte NetworkNumber[4];
  byte NodeAddress[6];
};

struct TNodeAddress {
  byte NodeAddress[6];
};

//=============================================================================
// struct TConnectionInfo
//
//=============================================================================

struct TConnectionInfo {
  byte ConnectStatus;
  byte Watchdog;
  word LocalConnectID;
  word RemoteConnectID;
  word SequenceNum;
  word LocalAcknowledgeNum;
  word LocalAllocationNum;
  word RemoteAcknowledgeNum;
  word RemoteAllocationNum;
  byte LocalSocket[2];
  byte ImmediateAddress[6];
  byte RemoteNetwork[4];
  byte RemoteNode[6];
  byte RemoteSocket[2];
  word RetransmitCOunt;
  word EstRoundtripTime;
  word RetransmittedPackets;
  word SuppressedPackets;
};

//============================================================================
// Function prototypes
// Both IPX and SPX
//============================================================================

// IPX functions
bool IPXInitialize(void);
word IPXCancelEvent(TEventControlBlock far *ecb);
void IPXCloseSocket(word socket);
void IPXDisconnectFromTarget(TIpxAddress far *rb);
void IPXGetInternetworkAddress(TNetworkAddress far *rb);
word IPXGetIntervalMarker(void);
word IPXGetLocalTarget(TIpxAddress far *rb, TNodeAddress far *na);
word IPXListenForPacket(TEventControlBlock far *ecb);
word IPXOpenSocket(word longevity, word *socket);
void IPXRelinquishControl(void);
void IPXSendPacket(TEventControlBlock far *ecb);

// SPX functions
// We are not using SPX here !!!
#if 0
void SPXAbortConnection(word connectionID);
word SPXInitialize(byte *major, byte *minor, word *maxconn, word *availconn);
word SPXEstablishConnection(word *connectionID, TEventControlBlock far *ecb,
                            byte retrycount, byte watchdog);
word SPXGetConnectionStatus(word connectionID, TConnectionInfo far *conninfo);
void SPXListenForConnection(TEventControlBlock far *ecb,
                            byte retrycount, byte watchdog);
void SPXListenForSequencedPacket(TEventControlBlock far *ecb);
void SPXSendSequencedPacket(word connectionID, TEventControlBlock far *ecb);
void SPXTerminateConnection(word connectionID, TEventControlBlock far *ecb);
#endif

#endif

/*#############################################################################
# End of file IPXSPX.H
#############################################################################*/

