/*#############################################################################
# MAIN.CPP
#
# 19.05.1997 (Codor)
#
# Program uses an IPX to perform various different tasks and
# functionilities which are useful when developing a program which uses
# an IPX without a Novell Network.
#############################################################################*/

#include <dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "types.h"
#include "video.h"
#include "utility.h"
#include "ipxspx.h"

static int PosX, PosY;

class TStation;

void usage(bool showall);
void showNodeAddress(void);
void execChecker(const char *name);
void _cdecl _far _loadds spyRxEventHandler(void);
void _cdecl _far _loadds spyTxEventHandler(void);
TStation *findStation(TStation *info);
void addStation(TStation *info);
void updateStation(TStation *station, TStation *info);
void sendBroadcast(void);
void updateScreen(void);

//=============================================================================
// Program entry point
//
//=============================================================================

void main(int argc, char *argv[])
{
  InitVideo();
  InitRoller();
  ScrollUp(1,1,80,25,25);
  PosX = PosY = 1;

  if (!TIpx::Init()) {
    usage(0);
    PutStrg(PosX,PosY,"IPX is not present on this machine!");
    char *info = "Press Ctrl-Break to stop ";
    PutStrg(1,25,info);
    CursorGoto(strlen(info)+1,25);
    while (bioskey(1) != -1)
      ;
    goto quit;
  }
  if (argc == 1) {
    usage(1);
    exit(0);
  } else if (argc == 2) {
    execChecker(argv[1]);
  }
  quit:
  ScrollUp(1,1,80,25,25);
  CursorGoto(1,1);
}

void usage(bool showall)
{
  PutStrg(1,2,"IPX Network Checker v1.000");
  PutStrg(1,3,"Copyright (c) 1999 SCA Software International S.A."
              " (http:\\www.scasoftware.com)");
  PutStrg(1,4,"Companion program for FESTO VIP driver for FESTO ViPWiN");
  PosY = 6;
  if (showall) {
    PutStrg(1,PosY++,"Usage: IPXCHK16 <computer_name>");
    PosY += 2;
  }
  CursorGoto(1,PosY);
}

void showNodeAddress(void)
{
  usage(0);
  //char *info = "Press any key to stop ";
  //PutStrg(1,25,info);
  //CursorGoto(strlen(info)+1,25);

  TNetworkAddress rb;
  char reverse[10];
  memset(&rb,'\0',sizeof(TNetworkAddress));
  IPXGetInternetworkAddress(&rb);
  char buf[81];
  sprintf(buf,"Network number ............: [%02X %02X %02X %02X]",
                  rb.NetworkNumber[0],rb.NetworkNumber[1],
                  rb.NetworkNumber[2],rb.NetworkNumber[3]);
  PutStrg(PosX,PosY++,buf);
  swab(&(rb.NetworkNumber[0]),&(reverse[0]),4);
  sprintf(buf,"Reversed network number ...: [%02X %02X %02X %02X]",
                  reverse[0],reverse[1],reverse[2],reverse[3]);
  PutStrg(PosX,PosY++,buf);
  sprintf(buf,"Node address ..............: [%02X %02X %02X %02X %02X %02X]",
                  rb.NodeAddress[0],rb.NodeAddress[1],
                  rb.NodeAddress[2],rb.NodeAddress[3],
                  rb.NodeAddress[4],rb.NodeAddress[5]);
  PutStrg(PosX,PosY++,buf);
  swab(&(rb.NodeAddress[0]),&(reverse[0]),6);
  sprintf(buf,"Reversed node address .....: [%02X %02X %02X %02X %02X %02X]",
                  reverse[0],reverse[1],reverse[2],
                  reverse[3],reverse[4],reverse[5]);
  PutStrg(PosX,PosY++,buf);

  char *info = "Press Ctrl-Break to stop ";
  PutStrg(1,25,info);
  CursorGoto(strlen(info)+1,25);
  while (bioskey(1) != -1)
    ;
}

//=============================================================================
// Field PC Net spy
//=============================================================================

struct TStation {
  bool Valid;
  char Name[64];
  byte Address[6];
  word Day;
  word Month;
  word Year;
  word Hour;
  word Minute;
  word Second;
};

struct TPacket {
  char Name[64];
  word Day;
  word Month;
  word Year;
  word Hour;
  word Minute;
  word Second;
};

#define STATION_LIST_SIZE   100
static TStation StationList[STATION_LIST_SIZE];
static char StationName[64];

static TIpxSocket SpySrc(0x5050,True);
static TIpx SpyRx(&SpySrc,spyRxEventHandler);
static char SpyBuffer[100];
static int SpyIndex= 5;

TIpxAddress SpyDestAddr(True);
TIpxSocket SpyDest(0x5050,True);
TIpx SpyTx(&SpyDest,spyTxEventHandler);

void execChecker(const char *name)
{
  usage(0);
  char *info = "Press Ctrl-Break to stop ";
  PutStrg(1,25,info);
  CursorGoto(strlen(info)+1,25);
  bool done = False;

  strcpy(StationName,name);
  strupr(StationName);

  for (int ndx = 0; ndx < STATION_LIST_SIZE; ndx ++)
    memset(&(StationList[ndx]),'\0',sizeof(TStation));

  TPeriod timer((word)1);
  SpySrc.Create();
  SpyRx.Listen();
  SpyDest.Create();
  SpyTx.SetDestination(&SpyDestAddr,&SpyDest);

  while (!done) {
    RefreshRoller(roUser1);
    if (timer.Expired()) {
      updateScreen();
      sendBroadcast();
      timer.Start((word)1);
    }
    done = False;
    if (bioskey(1) == -1)
      done = True;
  }

  SpyRx.Cancel();
  SpySrc.Destroy();
}

void _cdecl _far _loadds spyRxEventHandler(void)
{
  // ES:SI contains a pointer to ECB
  TEventControlBlock far *ecb = (TEventControlBlock far *)MK_FP(_ES,_SI);
  TPacket *packet = (TPacket*)ecb->Fragment[1].Address;
  TIpxHeader *header = (TIpxHeader*)ecb->Fragment[0].Address;

  TStation info;
  memset(&info,'\0',sizeof(TStation));
  info.Valid = True;
  strcpy(info.Name, packet->Name);
  memcpy(&(info.Address[0]),&(header->Src.Node[0]),6*sizeof(byte));
  info.Day = packet->Day;
  info.Month = packet->Month;
  info.Year = packet->Year;
  info.Hour = packet->Hour;
  info.Minute = packet->Minute;
  info.Second = packet->Second;

  TStation *station = findStation(&info);
  if (station == NULL) {
    addStation(&info);
  } else {
    updateStation(station, &info);
  }

  RefreshRoller(roUser2);
  SpyRx.Listen();
}

void _cdecl _far _loadds spyTxEventHandler(void)
{
  TEventControlBlock far *ecb = (TEventControlBlock far *)MK_FP(_ES,_SI);
  if (ecb->CompletionCode == TIpxStatus::EcbOk) {
    RefreshRoller(roUser3);
  }
}

void sendBroadcast(void)
{
  TDate d;
  TTime t;
  TPacket packet;
  memset(&packet,'\0', sizeof(TPacket));
  strcpy(packet.Name,StationName);
  packet.Day = d.Day();
  packet.Month = d.Month();
  packet.Year = d.Year();
  packet.Hour = t.Hour();
  packet.Minute = t.Min();
  packet.Second = t.Sec();

  SpyTx.Send(&packet,sizeof(TPacket));
}

TStation *findStation(TStation *info)
{
  for (int ndx = 0; ndx < STATION_LIST_SIZE; ndx ++) {
    if (StationList[ndx].Valid &&
        (memcmp(&(StationList[ndx].Address[0]),
               &(info->Address[0]), 6*sizeof(byte)) == 0))
    {
      return &(StationList[ndx]);
    }
  }
  return NULL;
}

void addStation(TStation *info)
{
  for (int ndx = 0; ndx < STATION_LIST_SIZE; ndx++) {
    if (!StationList[ndx].Valid)
      break;
  }
  memcpy(&(StationList[ndx]),info, sizeof(TStation));
}

void updateStation(TStation *station, TStation *info)
{
  station->Day = info->Day;
  station->Month = info->Month;
  station->Year = info->Year;
  station->Hour = info->Hour;
  station->Minute = info->Minute;
  station->Second = info->Second;
}

void updateScreen(void)
{
  BoxFill(1,6,80,17,' ');
  for (int ndx = 0; ndx < STATION_LIST_SIZE; ndx ++) {
    if (StationList[ndx].Valid) {
      if (strcmp(StationName,StationList[ndx].Name) == 0) {
        char name[61];
        sprintf(name,"%s (local)", StationList[ndx].Name);
        sprintf(SpyBuffer,"%02X:%02X:%02X:%02X:%02X:%02X   %-30s "
                          "%02d.%02d.%02d %02d:%02d:%02d",
                StationList[ndx].Address[0], StationList[ndx].Address[1],
                StationList[ndx].Address[2], StationList[ndx].Address[3],
                StationList[ndx].Address[4], StationList[ndx].Address[5],
                name,
                StationList[ndx].Day, StationList[ndx].Month,
                StationList[ndx].Year, StationList[ndx].Hour,
                StationList[ndx].Minute, StationList[ndx].Second);
      } else {
        sprintf(SpyBuffer,"%02X:%02X:%02X:%02X:%02X:%02X   %-30s "
                          "%02d.%02d.%02d %02d:%02d:%02d",
                StationList[ndx].Address[0], StationList[ndx].Address[1],
                StationList[ndx].Address[2], StationList[ndx].Address[3],
                StationList[ndx].Address[4], StationList[ndx].Address[5],
                StationList[ndx].Name,
                StationList[ndx].Day, StationList[ndx].Month,
                StationList[ndx].Year, StationList[ndx].Hour,
                StationList[ndx].Minute, StationList[ndx].Second);
      }
      PutStrg(1,6+ndx,SpyBuffer);
    }
  }
}

/*#############################################################################
# End of file MAIN.CPP
#############################################################################*/

