/*#############################################################################
# UTILITY.CPP
#
# 30.09.1995. Codor
#
# Description
# -----------
# Definition and member function of various useful classes
#############################################################################*/

#include <alloc.h>
#include <dir.h>
#include "utility.h"

//=============================================================================
// Utility functions
//
//=============================================================================

long hatoi(const char *s)
{
  long result = 0;
  char *ptr = (char*)s;
  if (*ptr == '0' && (*(ptr+1) == 'x' || *(ptr+1) == 'X')) {
    ptr += 2;
    while (ptr && *ptr && isxdigit(*ptr)) {
      uint i = *ptr++ - '0';
      if (9 < i)
        i -= 7;
      result <<= 4;
      result |= (i & 0x0F);
    }
  } else {
    result = atol(ptr);
  }
  return result;
}

bool isfile(char *name)
{
  struct ffblk block;
  return (findfirst(name,&block,FA_ARCH) == 0) ? True : False;
}

//=============================================================================
// Roller functionality
//
//=============================================================================

static char far *Video = NULL;
static char RollerCh[] = { '|', '/', '-', '\\' };
static int Rollers[10] = { 0,0,0,0,0,0,0,0,0,0 };

void InitRoller(void)
{
  union REGS r;
  r.h.ah = 0x0F;
  int86(0x10, &r, &r);
  switch (r.h.al) {
    case 7: Video = (char far *)0xB0000000L; break;
    case 2:
    case 3: Video = (char far *)0xB8000000L; break;
  }
}

void RefreshRoller(rollerId id)
{
  Video[id << 1] = RollerCh[Rollers[id]++];
  if (Rollers[id] >= 4)
    Rollers[id] = 0;
}

//=============================================================================
// class TPeriod
//
// This class is used to measure time intervals while some operation is being
// executed. It also supports the blank time interval. Minimal resolution is
// 1 second
//=============================================================================

TPeriod::TPeriod()
{
  working = False;
  timeOut = False;
  period = 0;
  stop = start = 0L;
}

TPeriod::TPeriod(long tickcount)
{
  Start(tickcount);
}

TPeriod::TPeriod(word secs)
{
  Start((long)(18.2*secs));
}

void TPeriod::Start(word secs)
{
  Start((long)(18.2*secs));
}

void TPeriod::Start(long tickcount)
{
  working = True;
  timeOut = False;
  period = tickcount;
  start = getTickCount();
}

//
// Return True if timeout has happened. Watch for rollback on midnight or
// somewhere else.
//
bool TPeriod::Expired(void)
{
  if (working) {
    stop = getTickCount();
    long diff = stop - start;
    if (diff < 0)
      diff = -diff;
    if (diff >= period) {
      timeOut = True;
      working = False;
      return True;
    }
  }
  return False;
}

//
// Return True if timeout
//
bool TPeriod::Timeout(void)
{
  return timeOut;
}

//
// Wait for the specified amount of time and do nothing.
//
void TPeriod::Wait(void)
{
  long now = getTickCount();
  while (getTickCount(now) - now < period)
    ;
}

//
// Returns the number of ticks the computer is running. Initial value is used
// to limit the lower level of ticks (clock() will return 0 if there is an
// error. If definite error is detected, do something (throw a exception)
//
long TPeriod::getTickCount(long /*lower*/)
{
  union {
    long ticks;
    word words[2];
  } count;
  union REGS inr, outr;
  inr.h.ah = 0x00;
  int86(0x1A, &inr, &outr);
  count.words[0] = outr.x.dx;
  count.words[1] = outr.x.cx;
  return count.ticks;
}

//=============================================================================
// class TDate
//
// It seems that something is wrong with a TDate class supplied with BC 4.5
// This class is less powerfull but works with our convention (dd.mm.yy)
//=============================================================================

TDate::TDate()
{
  Refresh();
  buff[0] = '\0';
}

TDate::TDate(unsigned char day, unsigned char mon, unsigned int year)
{
  d = day;
  m = mon;
  y = year;
}

TDate::TDate(const char *s)
{
  parse(s);
}

void TDate::Refresh(void)
{
  byte yBCDcent;
  byte yBCD;
  byte mBCD;
  byte dBCD;
  _AH = 4;
  asm int 0x1A;
  yBCDcent = _CH;
  yBCD = _CL;
  mBCD = _DH;
  dBCD = _DL;
  y = (yBCDcent >> 4) * 10;
  y += (yBCDcent & 0xF);
  y *= 100;
  y += (yBCD >> 4) * 10;
  y += (yBCD & 0xF);
  m = (mBCD >> 4) * 10;
  m += (mBCD & 0xF);
  d = (dBCD >> 4) * 10;
  d += (dBCD & 0xF);
}

char *TDate::AsString(prnFmt pf)
{
  char *ptr = buff;
  switch (pf) {
    case LongYear:  sprintf(buff, "%02d.%02d.%04d", d, m, y); break;
    case ShortYear: sprintf(buff, "%02d.%02d.%02d", d, m, y-1900); break;
    case NoYear:    sprintf(buff, "%02d.%02d", d, m); break;
  }
  return ptr;
}

void TDate::Assign(unsigned char td, unsigned char tm, unsigned int ty)
{
  d = td;
  m = tm;
  y = ty;
}

void TDate::Parse(const char *s)
{
  parse(s);
}

void TDate::parse(const char *s)
{
  unsigned char td, tm;
  unsigned int ty;
  char *pointPtr, *floatPtr = (char *) s;
  short len = strlen(s);

  d = m = y = 0;

  while (*floatPtr) {
    if (! ((*floatPtr >= '0' && *floatPtr <= '9') || *floatPtr == '.'))
      return;
    floatPtr ++;
  }

  floatPtr = (char *) s;
  pointPtr = strchr(floatPtr, '.');
  if (pointPtr - floatPtr == 2) {
    td = 10 * (*floatPtr-'0') + (*(floatPtr+1)-'0');
  } else if (pointPtr - floatPtr == 1) {
    td = (*floatPtr-'0');
  } else
    return;

  floatPtr = pointPtr + 1;
  pointPtr = strchr(floatPtr, '.');
  if (pointPtr - floatPtr == 2) {
    tm = 10 * (*floatPtr-'0') + (*(floatPtr+1)-'0');
  } else if (pointPtr - floatPtr == 1) {
    tm = (*floatPtr-'0');
  } else
    return;

  floatPtr = pointPtr + 1;
  pointPtr = (char *) &(s[len]);
  if (pointPtr - floatPtr == 2 || pointPtr - floatPtr == 4) {
    ty = atoi(floatPtr);
    if (pointPtr - floatPtr == 2)
      ty += 1900;
  } else
    return;

  d = td;
  m = tm;
  y = ty;
}

void TDate::operator= (const TDate &cd)
{
  d = cd.Day();
  m = cd.Month();
  y = cd.Year();
}

int TDate::operator==(const TDate& obj) const
{
  return (d == obj.Day() && m == obj.Month() && y == obj.Year());
}

void TDate::Set(void)
{
  byte yBCDcent, yBCD, mBCD, dBCD;
  yBCDcent = (y / 1000) << 4;    yBCDcent |= ((y % 1000) / 100);
  yBCD = (((y % 1000) % 100) / 10) << 4;  yBCD |= y % 10;
  mBCD = (m / 10) << 4; mBCD |= (m % 10);
  dBCD = (d / 10) << 4; dBCD |= (d % 10);
  _CH = yBCDcent;
  _CL = yBCD;
  _DH = mBCD;
  _DL = dBCD;
  _AH = 5;
  asm int 0x1A;
}

//=============================================================================
// class TTime
//
// There should be a time class when date class is already developed
//=============================================================================

TTime::TTime()
{
  Refresh();
  buff[0] = '\0';
}

TTime::TTime(unsigned char hour, unsigned char min, unsigned char sec)
{
  h = hour;
  m = min;
  s = sec;
}

TTime::TTime(const char *s)
{
  parse(s);
}

void TTime::Refresh(void)
{
  byte hBCD, mBCD, sBCD;
  _AH = 2;
  asm int 0x1A;
  hBCD = _CH;
  mBCD = _CL;
  sBCD = _DH;
  h = (hBCD >> 4) * 10;   h += (hBCD & 0xF);
  m = (mBCD >> 4) * 10;   m += (mBCD & 0xF);
  s = (sBCD >> 4) * 10;   s += (sBCD & 0xF);
}

char *TTime::AsString(prnFmt pf)
{
  char *ptr = buff;
  switch (pf) {
    case Full:  sprintf(buff, "%02d:%02d:%02d", h, m, s); break;
    case NoSec: sprintf(buff, "%02d:%02d", h, m); break;
  }
  return ptr;
}

void TTime::Assign(unsigned char th, unsigned char tm, unsigned char ts)
{
  h = th;
  m = tm;
  s = ts;
}

void TTime::Parse(const char *str)
{
  parse(str);
}

void TTime::parse(const char *str)
{
  unsigned char th,tm,ts;
  char *pointPtr, *floatPtr = (char*)str;
  short len = strlen(str);

  h = m = s = 0;

  while (*floatPtr) {
    if (! ((*floatPtr >= '0' && *floatPtr <= '9') || *floatPtr == ':'))
      return;
    floatPtr ++;
  }

  // Version 1.72
  // This is a bug. floatPtr must point to str.
  floatPtr = (char*)str;
  pointPtr = strchr(floatPtr, ':');
  if (pointPtr - floatPtr == 2) {
    th = 10 * (*floatPtr-'0') + (*(floatPtr+1)-'0');
  } else if (pointPtr - floatPtr == 1) {
    th = (*floatPtr-'0');
  } else
    return;

  floatPtr = pointPtr + 1;
  pointPtr = strchr(floatPtr, ':');
  if (pointPtr - floatPtr == 2) {
    tm = 10 * (*floatPtr-'0') + (*(floatPtr+1)-'0');
  } else if (pointPtr - floatPtr == 1) {
    tm = (*floatPtr-'0');
  } else
    return;

  floatPtr = pointPtr + 1;
  pointPtr = (char *) &(str[len]);
  if (pointPtr - floatPtr == 2) {
    ts = atoi(floatPtr);
  } else
    return;

  h = th;
  m = tm;
  s = ts;
}

void TTime::operator=(const TTime &ct)
{
  h = ct.Hour();
  m = ct.Min();
  s = ct.Sec();
}

int TTime::operator==(const TTime& obj) const
{
  return (h == obj.Hour() && m == obj.Min() && s == obj.Sec());
}

void TTime::Set(void)
{
  byte hBCD, mBCD, sBCD;
  hBCD = (h / 10) << 4;   hBCD |= (h % 10);
  mBCD = (m / 10) << 4;   mBCD |= (m % 10);
  sBCD = (s / 10) << 4;   sBCD |= (s % 10);
  _CH = hBCD;
  _CL = mBCD;
  _DH = sBCD;
  _AH = 3;
  asm int 0x1A;
}

//=============================================================================
// class TProfile
//
// Interface class to Windows like INI file.
//=============================================================================

TProfile::TProfile(char *fname, int len)
         :COMMENT(';'), LGROUP('['), RGROUP(']')
{
  LastError = None;
  strcpy(IniFile, fname);
  Len = len;
  Value = new char [Len+1];
  Line = new char [Len+1];
  Temp = new char [Len+1];
  Entry = new char [Len+1];
}

TProfile::~TProfile()
{
  delete [] Value;
  delete [] Line;
  delete [] Temp;
  delete [] Entry;
}

//
// Public member functions
// Open and close ini file. Return status and set internal error code
//
bool TProfile::Open(void)
{
  iniFP = fopen(IniFile, "rt");
  if (iniFP == NULL) {
    LastError = NoIniFile;
    return False;
  }
  setvbuf(iniFP, NULL, _IOFBF, 2048);
  LastError = None;
  return True;
}

void TProfile::Close(void)
{
  if (iniFP)
    fclose(iniFP);
  LastError = None;
}

//
// Enumerate a section
//
bool TProfile::Enum(char *section, EnumerateSection proc, dword data)
{
  bool retcode = False;
  word lnum = 0;
  if (locateSection(section, lnum)) {
    retcode = True;
    char entry[255], value[255];
    while (getEntry(lnum, entry, value)) {
      proc(section, entry, value, data);
    }
  } else
    LastError = NoSection;
  return retcode;
}

//
// Public member function
// Get the string value of the entry in the specified section. If the entry
// is not found, default string value is returned
//
char *TProfile::Get(char *section, char *entry, const char *deflt)
{
  word lnum = 0;
  if (locateSection(section, lnum)) {
    while (getEntry(lnum, Entry, Value)) {
      if (stricmp(Entry, entry) == 0) {
        LastError = None;
        Transient();
        return Value;
      }
    }
    LastError = NoEntry;
  } else
    LastError = NoSection;
  strcpy(Value, deflt);
  return Value;
}

//
// Public member function
// Get the integer value of the entry in the specified section. If the entry
// is not found, default integer value is returned.
//
int TProfile::Get(char *section, char *entry, int deflt)
{
  word lnum = 0;
  if (locateSection(section, lnum)) {
    while (getEntry(lnum, Entry, Value)) {
      if (stricmp(Entry, entry) == 0) {
        LastError = None;
        Transient();
        return atoi(Value);
      }
    }
    LastError = NoEntry;
  } else
    LastError = NoSection;
  return deflt;
}

//
// Public member function
// Get the long value of the entry in the specified section. If the entry
// is not found, default long value is returned.
//
long TProfile::Get(char *section, char *entry, long deflt)
{
  word lnum = 0;
  if (locateSection(section, lnum)) {
    while (getEntry(lnum, Entry, Value)) {
      if (stricmp(Entry, entry) == 0) {
        LastError = None;
        Transient();
        return atol(Value);
      }
    }
    LastError = NoEntry;
  } else
    LastError = NoSection;
  return deflt;
}

//
// This function is used to locate a section in the file. If the section is
// located, return code is True.
//
bool TProfile::locateSection(char *section, word& lnum)
{
  word status = lineOk;

  rewind(iniFP);
  strupr(section);
  lnum = 1;
  while (status != endOfFile) {
    status = readIniLine(Line);
    lnum += 1;
    strupr(Line);
    if (status == lineOk && isSection(Line) && stricmp(section,Line) == 0)
      return True;
  }
  return False;
}

//
// This function is used to find the entry inside the current section. If the
// entry is found, function returns True. Entry, value contain entry and its
// value
//
bool TProfile::getEntry(word& lnum, char *entry, char *value)
{
  word status = lineOk;

  while (status != endOfFile) {
    status = readIniLine(Line);
    lnum ++;
    if (status == lineOk) {
      strcpy(entry, Line);
      if (isEntry(entry, value))
        return True;
      else if (isSection(entry))
        return False;
    }
  }
  return False;
}

//
// Procitaj liniju iz ini datoteke i formatiraj je (ukloni komentar i
// whitespaces). Funkcija vraca (1) lineOk ako je linija ispravna,
// (2) endOfFile ako je kraj datoteke i (3) gotoNextLine ako treba citati
// sledecu liniju
//
TProfile::Status TProfile::readIniLine(char *li)
{
  char  *ptr,           /* Pomocni pointer */
        *lptr;          /* Pokazuje na prvi karakter u liniji */

  // Get the line from ini file
  if (fgets(Temp, Len-1, iniFP) == NULL)
    return endOfFile;    // End of file

  lptr = &(Temp[0]);

  // Remove whitespace from the start and '\n' at the end
  while (*lptr == ' ')
    lptr++;
  Temp[strlen(Temp)-1] = 0;

  // Remove comment from EOL
  if ((ptr = strchr(Temp, COMMENT)) != NULL)
    *ptr = 0;

  // Copy and skip empty lines
  strcpy(li, lptr);
  if (*lptr == 0)
    return gotoNextLine;

  // Remove whitespace
  ptr = &(li[0]);
  int skipwhite = True;
  while (*ptr != 0) {
    if (*ptr == '\"')
      skipwhite = (skipwhite) ? False : True;
    if (*ptr == '\"' || (skipwhite && *ptr == ' '))
      memmove(ptr, ptr+1, strlen(ptr));
    else
      ptr ++;
  }

  return lineOk;
}

//
// Functions receives the line and checks if the line contains the section
// name. If it does, section delimiters are removed and valid section name is
// returned
//
bool TProfile::isSection(char *li)
{
  word len = strlen(li);
  if (li[0] == LGROUP && li[len-1] == RGROUP) {
    char id[81];
    strcpy(id, &(li[1]));
    id[len-2] = 0;
    strcpy(li, id);
    return True;
  }
  return False;
}

//
// Functions receives line in the entry argument. It checks if the entry is
// a valid one. If it is, entry is split into valid entry and its value and
// function returns True. Otherwise, it returns False.
//
bool TProfile::isEntry(char *entry, char *value)
{
  char *ptr;
  if ((ptr = strchr(entry,'=')) == NULL)
    return False;
  *ptr = 0;                     // Exchange '=' for string terminator
  strcpy(value, ptr+1);         // Copy entry value
  return True;
}

//=============================================================================
// class TDebug
// Debug class that saves the debug messages into the text file that is
// created on each program start. If _DEBUG macro is not defined, the program
// will create an object of this class but will not access the storage (debug
// file will not be created). Version 1.41.
//=============================================================================

void TRACE(char *fmt, ...)
{
  #ifdef _DEBUG
  char temp[81];
  va_list argptr;
  va_start(argptr, fmt);
  vsprintf(temp, fmt, argptr);
  va_end(argptr);
  printf("%s\n",temp);
  #else
  fmt;      // Prevent compiler warning
  #endif
}

TDebug::TDebug(const char *name, bool logtime)
{
  #ifdef _DEBUG
  logTime = logtime;
  dbgFile = fopen(name,"wt");
  if (dbgFile) {
    TDate d;
    TTime t;
    fprintf(dbgFile,"\n>>>\n>>> Debug session started [%s,%s].\n>>>\n",
                    d.AsString(),t.AsString());
  }
  #else
  name;
  #endif
}

TDebug::~TDebug()
{
  #ifdef _DEBUG
  if (dbgFile) {
    TDate d;
    TTime t;
    fprintf(dbgFile,"<<<\n<<< Debug session ended [%s,%s].\n<<<\n",
                    d.AsString(),t.AsString());
    fclose(dbgFile);
  }
  #endif
}

void TDebug::Log(char *string, ...)
{
  #ifdef _DEBUG
  if (dbgFile) {
    char temp[255];
    va_list argptr;
    va_start(argptr, string);
    vsprintf(temp, string, argptr);
    va_end(argptr);
    if (logTime) {
      fprintf(dbgFile,"%s,%s : %s\n", TDate().AsString(TDate::ShortYear),
                                      TTime().AsString(TTime::Full),
                                      temp);
    } else {
      fprintf(dbgFile,"%s\n", temp);
    }
  }
  #else
  string;
  #endif
}

void TDebug::Show(char *string, ...)
{
  #ifdef _DEBUG
  char temp[255];
  va_list argptr;
  va_start(argptr, string);
  vsprintf(temp, string, argptr);
  va_end(argptr);
  printf("%s\n",temp);
  #else
  string;
  #endif
}

//=============================================================================
// class TDebugMemory
// Object of this class must be declared as static and outside of main()
// program function. It should also be the first object created.
// Version 1.41. If _DEBUG macro is not defined, object of this class is
// created but does nothing.
//=============================================================================

TDebugMemory::TDebugMemory()
{
  #ifdef _DEBUG
  MemB = coreleft();
  #endif
}

TDebugMemory::~TDebugMemory()
{
  #ifdef _DEBUG
  MemA = coreleft();
  printf("\n\nMemory status = %lu : %lu\n\n",MemB,MemA);
  #endif
}

/*#############################################################################
# End of UTILITY.CPP
#############################################################################*/
